@@ -5,7 +5,7 @@ import kaa
 from kaa.command import Commands, commandid, is_enable, norec, norerun
 from kaa import document
 from kaa.filetype.default import modebase
-from gappedbuf import re as gre
+from kaa import doc_re
 
 
 class CursorCommands(Commands):
@@ -14,53 +14,58 @@ class CursorCommands(Commands):
     @norerun
     def right(self, wnd):
         if wnd.screen.selection.has_mark():
-            wnd.cursor.right()
+            for i in range(wnd.editmode.get_repeat()):
+                wnd.cursor.right()
         elif wnd.screen.selection.is_selected():
-            range = wnd.screen.selection.get_selrange()
-            if wnd.cursor.pos > range[1]:
+            selrange = wnd.screen.selection.get_selrange()
+            if wnd.cursor.pos > selrange[1]:
                 wnd.cursor.right()
             else:
                 wnd.cursor.setpos(wnd.cursor.adjust_nextpos(
-                    wnd.cursor.pos, range[1]))
+                    wnd.cursor.pos, selrange[1]))
         else:
             wnd.document.mode.cancel_auto_indent(wnd)
-            wnd.cursor.right()
+            for i in range(wnd.editmode.get_repeat()):
+                wnd.cursor.right()
 
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.right.select')
     @norerun
     def right_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.right()
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.right()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.left')
     @norerun
     def left(self, wnd):
         if wnd.screen.selection.has_mark():
-            wnd.cursor.left()
+            for i in range(wnd.editmode.get_repeat()):
+                wnd.cursor.left()
         elif wnd.screen.selection.is_selected():
-            range = wnd.screen.selection.get_selrange()
-            if wnd.cursor.pos < range[0]:
+            selrange = wnd.screen.selection.get_selrange()
+            if wnd.cursor.pos < selrange[0]:
                 wnd.cursor.left()
             else:
                 wnd.cursor.setpos(wnd.cursor.adjust_nextpos(
-                    wnd.cursor.pos, range[0]))
+                    wnd.cursor.pos, selrange[0]))
         else:
             wnd.document.mode.cancel_auto_indent(wnd)
-            wnd.cursor.left()
+            for i in range(wnd.editmode.get_repeat()):
+                wnd.cursor.left()
 
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.left.select')
     @norerun
     def left_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
         wnd.cursor.left()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -68,119 +73,133 @@ class CursorCommands(Commands):
     @norerun
     def up(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.up()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.up()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.up.select')
     @norerun
     def up_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.up()
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.up()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.down')
     @norerun
     def down(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.down()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.down()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.down.select')
     @norerun
     def down_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.down()
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.down()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.prev-line')
     @norerun
     def prev_line(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.prev_line()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.prev_line()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.next-line')
     @norerun
     def next_line(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.next_line()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.next_line()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.word-right')
     @norerun
     def word_right(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.right(word=True)
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.right(word=True)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.word-right.select')
     @norerun
     def word_right_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.right(word=True)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.right(word=True)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.word-left')
     @norerun
     def word_left(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.left(word=True)
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.left(word=True)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.word-left.select')
     @norerun
     def word_left_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.left(word=True)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.left(word=True)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.pagedown')
     @norerun
     def pagedown(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.pagedown()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.pagedown()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.pagedown.select')
     @norerun
     def pagedown_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.pagedown()
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.pagedown()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.pageup')
     @norerun
     def pageup(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
-        wnd.cursor.pageup()
+        wnd.screen.selection.end_cursor_sel()
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.pageup()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.pageup.select')
     @norerun
     def pageup_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
-        wnd.cursor.pageup()
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.pageup()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
     @commandid('cursor.home')
     @norerun
     def home(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.home()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -188,7 +207,7 @@ class CursorCommands(Commands):
     @norerun
     def home_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
         wnd.cursor.home()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -196,7 +215,7 @@ class CursorCommands(Commands):
     @norerun
     def end(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.end()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -204,7 +223,7 @@ class CursorCommands(Commands):
     @norerun
     def end_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
         wnd.cursor.end()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -212,7 +231,7 @@ class CursorCommands(Commands):
     @norerun
     def tol(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.tol(wnd.cursor.pos)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -220,7 +239,7 @@ class CursorCommands(Commands):
     @norerun
     def first_letter(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.first_letter(wnd.cursor.pos)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -228,7 +247,7 @@ class CursorCommands(Commands):
     @norerun
     def eol(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.eol(wnd.cursor.pos)
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -236,7 +255,7 @@ class CursorCommands(Commands):
     @norerun
     def top(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.tof()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -244,7 +263,7 @@ class CursorCommands(Commands):
     @norerun
     def top_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
         wnd.cursor.tof()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -252,7 +271,7 @@ class CursorCommands(Commands):
     @norerun
     def last(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
         wnd.cursor.eof()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -260,7 +279,7 @@ class CursorCommands(Commands):
     @norerun
     def last_select(self, wnd):
         wnd.document.mode.cancel_auto_indent(wnd)
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
         wnd.cursor.eof()
         wnd.screen.selection.set_to(wnd.cursor.pos)
 
@@ -281,7 +300,7 @@ class CursorCommands(Commands):
                 return
 
             wnd.document.mode.cancel_auto_indent(wnd)
-            wnd.screen.selection.end_cursor()
+            wnd.screen.selection.end_cursor_sel()
 
             pos = wnd.document.get_lineno_pos(lineno)
             tol = wnd.document.gettol(pos)
@@ -300,7 +319,7 @@ class SelectionCommands(Commands):
     @commandid('selection.begin-cursor')
     @norerun
     def selection_begin(self, wnd):
-        wnd.screen.selection.begin_cursor(wnd.cursor.pos)
+        wnd.screen.selection.begin_cursor_sel(wnd.cursor.pos)
 
     @commandid('selection.set-to')
     @norerun
@@ -310,7 +329,7 @@ class SelectionCommands(Commands):
     @commandid('selection.end-cursor')
     @norerun
     def selection_clear(self, wnd):
-        wnd.screen.selection.end_cursor()
+        wnd.screen.selection.end_cursor_sel()
 
     @commandid('selection.set-mark')
     @norerun
@@ -320,6 +339,11 @@ class SelectionCommands(Commands):
         else:
             wnd.screen.selection.set_mark(None)
 
+    @commandid('selection.set-linewise-mark')
+    @norerun
+    def selection_set_linewisemark(self, wnd):
+        wnd.screen.selection.set_linewise_mark(wnd.cursor.pos)
+
     @commandid('selection.set-rectangle-mark')
     @norerun
     def selection_set_rectangle_mark(self, wnd):
@@ -386,7 +410,7 @@ class SelectionCommands(Commands):
     def lineselection_begin(self, wnd):
         tol = wnd.cursor.adjust_nextpos(
             wnd.cursor.pos, wnd.document.gettol(wnd.cursor.pos))
-        wnd.screen.selection.begin_cursor(tol)
+        wnd.screen.selection.begin_cursor_sel(tol)
 
     @commandid('selection.line.set-end')
     @norerun
@@ -421,9 +445,7 @@ class EditCommands(Commands):
                 return self._delete_rect_sel(wnd)
 
     def _delete_rect_sel(self, wnd):
-        if wnd.document.undo:
-            wnd.document.undo.beginblock()
-        try:
+        with wnd.document.undo_group():
             (posfrom, posto, colfrom, colto
              ) = wnd.screen.selection.get_rect_range()
 
@@ -439,17 +461,16 @@ class EditCommands(Commands):
                 posfrom = wnd.document.geteol(posfrom)
 
             return True
-        finally:
-            if wnd.document.undo:
-                wnd.document.undo.endblock()
 
     @commandid('edit.delete')
     def delete(self, wnd):
         if self.delete_sel(wnd):
             return
 
-        pos = wnd.cursor.pos
-        nextpos = wnd.document.get_nextpos(pos)
+        pos = nextpos = wnd.cursor.pos
+        for i in range(wnd.editmode.get_repeat()):
+            nextpos = wnd.document.get_nextpos(nextpos)
+
         nextpos = wnd.cursor.adjust_nextpos(pos, nextpos)
         if pos < nextpos:
             wnd.document.mode.delete_string(wnd, pos, nextpos)
@@ -460,7 +481,9 @@ class EditCommands(Commands):
             return
 
         pos = wnd.cursor.pos
-        wnd.cursor.right(word=True)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.right(word=True)
+
         nextpos = wnd.cursor.pos
         if pos < nextpos:
             wnd.document.mode.delete_string(wnd, pos, nextpos)
@@ -469,15 +492,16 @@ class EditCommands(Commands):
     def delete_line(self, wnd):
         pos = wnd.cursor.pos
         nextpos = wnd.cursor.adjust_nextpos(
-            pos, wnd.document.find_newline(pos))
+            pos, wnd.document.get_line_to(pos))
         if pos < nextpos:
             wnd.document.mode.delete_string(wnd, pos, nextpos)
 
     @commandid('edit.delete.currentline')
     def delete_currentline(self, wnd):
         pos = wnd.cursor.pos
-        f = wnd.cursor.adjust_nextpos(pos, wnd.document.gettol(pos))
-        t = wnd.cursor.adjust_nextpos(pos, wnd.document.geteol(f))
+        f = t = wnd.cursor.adjust_nextpos(pos, wnd.document.gettol(pos))
+        for i in range(wnd.editmode.get_repeat()):
+            t = wnd.cursor.adjust_nextpos(pos, wnd.document.geteol(t))
 
         if f < t:
             wnd.document.mode.delete_string(wnd, f, t)
@@ -487,9 +511,11 @@ class EditCommands(Commands):
         if self.delete_sel(wnd):
             return
 
-        pos = wnd.cursor.pos
-        prevpos = wnd.cursor.adjust_nextpos(
-            pos, wnd.document.get_prevpos(pos))
+        pos = prevpos = wnd.cursor.pos
+        for i in range(wnd.editmode.get_repeat()):
+            prevpos = wnd.cursor.adjust_nextpos(
+                pos, wnd.document.get_prevpos(prevpos))
+
         if prevpos < pos:
             if pos == wnd.screen.pos:
                 # locate cursor before delete to scroll half page up
@@ -503,7 +529,9 @@ class EditCommands(Commands):
             return
 
         pos = wnd.cursor.pos
-        wnd.cursor.left(word=True)
+        for i in range(wnd.editmode.get_repeat()):
+            wnd.cursor.left(word=True)
+
         prevpos = wnd.cursor.pos
         if prevpos < pos:
             if prevpos < wnd.screen.pos:
@@ -545,33 +573,29 @@ class EditCommands(Commands):
             self._indent_line(wnd, wnd.cursor.pos)
             return
 
-        doc = wnd.document
-        tol, eol = doc.mode.get_line_sel(wnd)
-        wnd.screen.selection.set_range(tol, eol)
+        for i in range(wnd.editmode.get_repeat()):
+            doc = wnd.document
+            tol, eol = doc.mode.get_line_sel(wnd)
+            wnd.screen.selection.set_range(tol, eol)
 
-        if wnd.document.undo:
-            wnd.document.undo.beginblock()
-        try:
-            mode = wnd.document.mode
-            while tol < wnd.screen.selection.get_end():
-                f, t = mode.get_indent_range(tol)
-                if f != t:
-                    cols = mode.calc_cols(f, t)
-                else:
-                    cols = 0
+            with wnd.document.undo_group():
+                mode = wnd.document.mode
+                while tol < wnd.screen.selection.get_selrange()[1]:
+                    f, t = mode.get_indent_range(tol)
+                    if f != t:
+                        cols = mode.calc_cols(f, t)
+                    else:
+                        cols = 0
 
-                endpos = doc.endpos()
-                eol = doc.geteol(tol)
+                    endpos = doc.endpos()
+                    eol = doc.geteol(tol)
 
-                if (t + 1 < eol != endpos) or (t < eol == endpos):
-                    s = mode.build_indent_str(cols + mode.indent_width)
-                    mode.replace_string(wnd, f, t, s, False)
-                    tol = eol + (len(s) - (t - f))
-                else:
-                    tol = eol
-        finally:
-            if wnd.document.undo:
-                wnd.document.undo.endblock()
+                    if (t + 1 < eol != endpos) or (t < eol == endpos):
+                        s = mode.build_indent_str(cols + mode.indent_width)
+                        mode.replace_string(wnd, f, t, s, False)
+                        tol = eol + (len(s) - (t - f))
+                    else:
+                        tol = eol
 
         f, t = wnd.screen.selection.get_selrange()
         wnd.cursor.setpos(f)
@@ -594,29 +618,26 @@ class EditCommands(Commands):
             self._dedent_line(wnd, wnd.cursor.pos)
             return
 
-        doc = wnd.document
-        tol, eol = doc.mode.get_line_sel(wnd)
-        wnd.screen.selection.set_range(tol, eol)
+        for i in range(wnd.editmode.get_repeat()):
+            doc = wnd.document
+            tol, eol = doc.mode.get_line_sel(wnd)
+            wnd.screen.selection.set_range(tol, eol)
 
-        if wnd.document.undo:
-            wnd.document.undo.beginblock()
-        try:
-            mode = wnd.document.mode
-            while tol < wnd.screen.selection.get_end():
-                f, t = mode.get_indent_range(tol)
-                if f != t:
-                    cols = mode.calc_cols(f, t)
-                else:
-                    cols = 0
+            with wnd.document.undo_group():
+                mode = wnd.document.mode
+                while tol < wnd.screen.selection.get_selrange()[1]:
+                    f, t = mode.get_indent_range(tol)
+                    if f != t:
+                        cols = mode.calc_cols(f, t)
+                    else:
+                        cols = 0
 
-                if cols:
-                    s = mode.build_indent_str(max(0, cols - mode.indent_width))
-                    mode.replace_string(wnd, f, t, s, False)
+                    if cols:
+                        s = mode.build_indent_str(
+                            max(0, cols - mode.indent_width))
+                        mode.replace_string(wnd, f, t, s, False)
 
-                tol = doc.geteol(tol)
-        finally:
-            if wnd.document.undo:
-                wnd.document.undo.endblock()
+                    tol = doc.geteol(tol)
 
         f, t = wnd.screen.selection.get_selrange()
         wnd.cursor.setpos(f)
@@ -641,17 +662,18 @@ class EditCommands(Commands):
     @commandid('edit.undo')
     @norerun
     def undo(self, wnd):
-        if wnd.document.undo and wnd.document.undo.can_undo():
-            wnd.screen.selection.clear()
-            pos = None
-            for rec in wnd.document.undo.undo():
-                pos = self._undo(wnd, rec)
+        for i in range(wnd.editmode.get_repeat()):
+            if wnd.document.undo and wnd.document.undo.can_undo():
+                wnd.screen.selection.clear()
+                pos = None
+                for rec in wnd.document.undo.undo():
+                    pos = self._undo(wnd, rec)
 
-            if pos is not None:
-                wnd.cursor.setpos(pos)
-                wnd.cursor.savecol()
+                if pos is not None:
+                    wnd.cursor.setpos(pos)
+                    wnd.cursor.savecol()
 
-            wnd.document.mode.on_edited(wnd)
+                wnd.document.mode.on_edited(wnd)
 
     def _redo(self, wnd, rec):
         (action, args, kwargs) = rec
@@ -672,17 +694,18 @@ class EditCommands(Commands):
     @commandid('edit.redo')
     @norerun
     def redo(self, wnd):
-        if wnd.document.undo and wnd.document.undo.can_redo():
-            wnd.screen.selection.clear()
-            pos = None
-            for rec in wnd.document.undo.redo():
-                pos = self._redo(wnd, rec)
+        for i in range(wnd.editmode.get_repeat()):
+            if wnd.document.undo and wnd.document.undo.can_redo():
+                wnd.screen.selection.clear()
+                pos = None
+                for rec in wnd.document.undo.redo():
+                    pos = self._redo(wnd, rec)
 
-            if pos is not None:
-                wnd.cursor.setpos(pos)
-                wnd.cursor.savecol()
+                if pos is not None:
+                    wnd.cursor.setpos(pos)
+                    wnd.cursor.savecol()
 
-            wnd.document.mode.on_edited(wnd)
+                wnd.document.mode.on_edited(wnd)
 
     def _get_sel(self, wnd):
         if wnd.screen.selection.is_selected():
@@ -727,8 +750,10 @@ class EditCommands(Commands):
     def paste(self, wnd):
         s = kaa.app.clipboard.get()
         if s:
-            wnd.document.mode.put_string(wnd, s)
-            wnd.screen.selection.clear()
+            with wnd.document.undo_group():
+                for i in range(wnd.editmode.get_repeat()):
+                    wnd.document.mode.put_string(wnd, s)
+                    wnd.screen.selection.clear()
 
     @commandid('edit.conv.upper')
     def conv_upper(self, wnd):
@@ -800,27 +825,22 @@ class CodeCommands(Commands):
             tol, eol = wnd.document.mode.get_line_sel(wnd)
             wnd.screen.selection.set_range(tol, eol)
 
-            if wnd.document.undo:
-                wnd.document.undo.beginblock()
-            try:
+            with wnd.document.undo_group():
                 mode = wnd.document.mode
-                while tol < wnd.screen.selection.get_end():
+                while tol < wnd.screen.selection.get_selrange()[1]:
                     wnd.document.mode.insert_string(
                         wnd, tol, wnd.document.mode.LINE_COMMENT,
                         update_cursor=False)
                     tol = wnd.document.geteol(tol)
-            finally:
-                if wnd.document.undo:
-                    wnd.document.undo.endblock()
 
             f, t = wnd.screen.selection.get_selrange()
             wnd.cursor.setpos(f)
             wnd.cursor.savecol()
 
     def _is_comment_line(self, wnd, pos):
-        reobj = gre.compile(r'[ \t]*({})'.format(
-            gre.escape(wnd.document.mode.LINE_COMMENT)))
-        return reobj.match(wnd.document.buf, pos)
+        reobj = doc_re.compile(r'[ \t]*({})'.format(
+            doc_re.escape(wnd.document.mode.LINE_COMMENT)))
+        return reobj.match(wnd.document, pos)
 
     @commandid('code.region.unlinecomment')
     def uncomment(self, wnd):
@@ -836,20 +856,15 @@ class CodeCommands(Commands):
             tol, eol = wnd.document.mode.get_line_sel(wnd)
             wnd.screen.selection.set_range(tol, eol)
 
-            if wnd.document.undo:
-                wnd.document.undo.beginblock()
-            try:
+            with wnd.document.undo_group():
                 mode = wnd.document.mode
-                while tol < wnd.screen.selection.get_end():
+                while tol < wnd.screen.selection.get_selrange()[1]:
                     m = self._is_comment_line(wnd, tol)
                     if m:
                         f, t = m.span(1)
                         wnd.document.mode.delete_string(
                             wnd, f, t, update_cursor=False)
                     tol = wnd.document.geteol(tol)
-            finally:
-                if wnd.document.undo:
-                    wnd.document.undo.endblock()
 
             f, t = wnd.screen.selection.get_selrange()
             wnd.cursor.setpos(f)
@@ -864,8 +879,7 @@ class SearchCommands(Commands):
     def showsearch(self, wnd):
         from kaa.ui.searchdlg import searchdlgmode
 
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = searchdlgmode.SearchDlgMode(target=wnd)
         doc.setmode(mode)
         mode.build_document()
@@ -878,8 +892,7 @@ class SearchCommands(Commands):
     def showreplace(self, wnd):
         from kaa.ui.searchdlg import searchdlgmode
 
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = searchdlgmode.ReplaceDlgMode(target=wnd)
         doc.setmode(mode)
         mode.build_document()
@@ -937,6 +950,7 @@ class SearchCommands(Commands):
 
         if not ret:
             if start != wnd.document.endpos():
-                ret = wnd.document.mode.search_prev(wnd.document.endpos(),
-                                                    modebase.SearchOption.LAST_SEARCH)
+                ret = wnd.document.mode.search_prev(
+                    wnd.document.endpos(),
+                    modebase.SearchOption.LAST_SEARCH)
                 self._show_searchresult(wnd, ret)
