@@ -19,7 +19,7 @@ class CuiApp:
     DEFAULT_THEME = 'basic'
     DEFAULT_PALETTE = 'dark'
     NUM_NEWFILE = 1
-    
+
     def __init__(self, config):
         self.config = config
         self.clipboard = clipboard.select_clipboard()
@@ -72,8 +72,7 @@ class CuiApp:
         from kaa.ui.messagebar import messagebarmode
         self.messagebar = messagebarmode.MessageBarMode()
 
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         doc.setmode(self.messagebar)
         mainframe.set_messagebar(doc)
 
@@ -262,7 +261,7 @@ class CuiApp:
 
     def dump_panel(self):
         import curses.panel
-        panels =[]
+        panels = []
         p = curses.panel.top_panel()
         while p:
             panels.append(p)
@@ -277,7 +276,6 @@ class CuiApp:
         _trace('>>>>>>>>>>>>>>>>>>>>>>')
         _trace(panels)
 
-
     def run(self):
 #        def f(t, i):
 #            _trace(t, i)
@@ -291,6 +289,7 @@ class CuiApp:
                     kaa.log.error('Internal error: invalid focus window.')
                     break
 
+                self.focus.restore_cursor_pos()
                 if not nonblocking:
                     # update screen before sleep.
                     curses.panel.update_panels()
