@@ -1,3 +1,4 @@
+import contextlib
 import weakref
 import re
 import gappedbuf
@@ -174,6 +175,14 @@ class Document:
         else:
             self.undo = None
 
+    @contextlib.contextmanager
+    def undo_group(self):
+        if self.undo:
+            self.undo.beginblock()
+        yield None
+        if self.undo:
+            self.undo.endblock()
+
     def set_title(self, title):
         self.title = title
 
@@ -244,7 +253,7 @@ class Document:
         tol = self.buf.rfindchr('\n', 0, pos)
         return 0 if tol == -1 else tol + 1
 
-    def find_newline(self, pos):
+    def get_line_to(self, pos):
         eol = self.buf.findchr('\n', pos, len(self.buf))
         if eol == -1:
             eol = len(self.buf)
