@@ -3,7 +3,7 @@ import itertools
 import unicodedata
 import collections
 
-import gappedbuf.re
+from kaa import doc_re
 import kaa
 from kaa import keyboard, editmode
 import kaa.log
@@ -12,8 +12,9 @@ from kaa import theme
 from kaa import screen
 from kaa import addon
 
+
 class SearchOption:
-    RE = gappedbuf.re
+    RE = doc_re
 
     def __init__(self):
         self.text = ''
@@ -202,7 +203,7 @@ class ModeBase:
                 self.keybind_vi_visuallinewisemode.add_keybind(keys)
 
     def init_commands(self):
-        from kaa.commands import (editorcommand, editmodecommand)
+        from kaa.commands import (editorcommand, editmodecommand, vicommand)
 
         self.register_commandobj(editorcommand.CursorCommands())
         self.register_commandobj(editorcommand.EditCommands())
@@ -210,6 +211,7 @@ class ModeBase:
         self.register_commandobj(editorcommand.SelectionCommands())
         self.register_commandobj(editorcommand.SearchCommands())
         self.register_commandobj(editmodecommand.EditModeCommands())
+        self.register_commandobj(vicommand.ViCommands())
 
         for name in dir(self):
             attr = getattr(self, name)
@@ -251,6 +253,9 @@ class ModeBase:
     def editmode_insert(self, wnd):
         wnd.set_editmode(editmode.EditMode())
 
+    def editmode_replace(self, wnd):
+        wnd.set_editmode(editmode.ReplaceMode())
+
     def editmode_visual(self, wnd):
         wnd.set_editmode(editmode.VisualMode())
 
@@ -312,8 +317,8 @@ class ModeBase:
     def on_edited(self, wnd):
         pass
 
-    def on_str(self, wnd, s):
-        self.put_string(wnd, s)
+    def on_str(self, wnd, s, overwrite=False):
+        self.put_string(wnd, s, overwrite=overwrite)
         wnd.screen.selection.clear()
 
         if kaa.app.macro.is_recording():
@@ -415,9 +420,7 @@ class ModeBase:
             self.on_edited(wnd)
 
     def replace_rect(self, wnd, repto):
-        if wnd.document.undo:
-            wnd.document.undo.beginblock()
-        try:
+        with wnd.document.undo_group():
             (posfrom, posto, colfrom, colto
              ) = wnd.screen.selection.get_rect_range()
 
@@ -434,12 +437,9 @@ class ModeBase:
                     self.replace_string(wnd, f, t, s)
                     posto += (len(s) - (t - f))
                 posfrom = wnd.document.geteol(posfrom)
-        finally:
-            if wnd.document.undo:
-                wnd.document.undo.endblock()
 
-    def put_string(self, wnd, s):
-        s = wnd.document.mode.filter_string(wnd, s)
+    def put_string(self, wnd, s, overwrite=False):
+        s = self.filter_string(wnd, s)
 
         if wnd.screen.selection.is_selected():
             if wnd.screen.selection.is_rectangular():
@@ -453,7 +453,12 @@ class ModeBase:
                 f, t = sel
                 self.replace_string(wnd, f, t, s)
         else:
-            self.insert_string(wnd, wnd.cursor.pos, s)
+            if not overwrite:
+                self.insert_string(wnd, wnd.cursor.pos, s)
+            else:
+                eol = wnd.document.get_line_to(wnd.cursor.pos)
+                posto = min(wnd.cursor.pos + len(s), eol)
+                self.replace_string(wnd, wnd.cursor.pos, posto, s)
 
     def update_charattr(self, wnd):
         if wnd.charattrs:
@@ -495,13 +500,13 @@ class ModeBase:
     RE_HIRAGANA = r"(?P<L_HIRAGANA>[\u3040-\u309f\u30fc]+)"
     RE_KATAKANA = r"(?P<L_KATAKANA>[\u30a0-\u30ff\u30fc]+)"
 
-    RE_SPLITWORD = gappedbuf.re.compile('|'.join((
+    RE_SPLITWORD = doc_re.compile('|'.join((
         RE_WORDCHAR, RE_WHITESPACE, RE_LF, RE_HIRAGANA, RE_KATAKANA)))
 
     def split_word(self, begin, all=False):
         """yield word in the document until line ends"""
 
-        for m in self.RE_SPLITWORD.finditer(self.document.buf, pos=begin):
+        for m in self.RE_SPLITWORD.finditer(self.document, pos=begin):
             # split unmatched characters by character category.
             f, t = m.span()
             if f != begin:
@@ -552,13 +557,13 @@ class ModeBase:
     def search_next(self, pos, searchinfo):
         regex = searchinfo.get_regex()
         pos = min(max(0, pos), self.document.endpos())
-        m = regex.search(self.document.buf, pos)
+        m = regex.search(self.document, pos)
         return m
 
     def search_prev(self, pos, searchinfo):
         regex = searchinfo.get_regex()
         last = None
-        for m in regex.finditer(self.document.buf, 0):
+        for m in regex.finditer(self.document, 0):
             span = m.span()
             if span[1] >= pos:
                 break
@@ -585,8 +590,8 @@ class ModeBase:
 
     def get_indent_range(self, pos):
         tol = self.document.gettol(pos)
-        regex = gappedbuf.re.compile(self.RE_WHITESPACE)
-        m = regex.match(self.document.buf, tol)
+        regex = doc_re.compile(self.RE_WHITESPACE)
+        m = regex.match(self.document, tol)
         if m:
             return m.span()
         else:
