@@ -39,8 +39,7 @@ class WordCompleteInputMode(filterlist.FilterListInputDlgMode):
 
     @classmethod
     def build(cls, target):
-        buf = document.Buffer()
-        doc = document.Document(buf)
+        doc = document.Document()
         mode = cls()
         doc.setmode(mode)
         mode.target = target
